#include "../common/cleanup.h"
#include "../common/const.h"
#include "../common/global.h"
#include "../common/tls.h"
#include <arpa/inet.h>
#include <csignal>
#include <iostream>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <sys/socket.h>
#include <unistd.h>
SSL_CTX *ctx;
int sockfd;

int main() {
  std::signal(SIGTERM, safe_exit);
  std::signal(SIGINT, safe_exit);
  std::signal(SIGQUIT, safe_exit);
  std::signal(SIGHUP, safe_exit);

  // error signal handlers
  signal(SIGSEGV, safe_exit);
  ctx = create_context(false);
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    std::cerr << "Socket creation failed: " << strerror(errno) << std::endl;
    return 1;
  }
  std::cout << "Socket created successfully (fd: " << sockfd << ")"
            << std::endl;

  sockaddr_in serv_addr{};
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(PORT);
  inet_pton(AF_INET, "127.0.0.1", &serv_addr.sin_addr);

  if (connect(sockfd, (sockaddr *)&serv_addr, sizeof(serv_addr)) != 0) {
    std::cerr << "Connection failed: " << strerror(errno) << std::endl;
    return 1;
  }

  SSL *ssl = SSL_new(ctx);
  SSL_set_fd(ssl, sockfd);
  int result = SSL_connect(ssl);
  if (result != 1) {
    int ssl_error = SSL_get_error(ssl, result);
    std::cerr << "SSL_connect failed with error code: " << ssl_error
              << std::endl;
    ERR_print_errors_fp(stderr);
    return 1;
  }
  std::cout << "SSL connection established successfully" << std::endl;

  handle_connection(ssl);

  SSL_shutdown(ssl);
  SSL_free(ssl);
  close(sockfd);
  SSL_CTX_free(ctx);
  return 0;
}
