#include "const.h"
#include <iostream>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <string>
#include <thread>

SSL_CTX *create_context(bool is_server) {
  const SSL_METHOD *method =
      is_server ? TLS_server_method() : TLS_client_method();
  SSL_CTX *ctx = SSL_CTX_new(method);
  SSL_CTX_set_min_proto_version(ctx, TLS1_3_VERSION);

  if (is_server) {
    SSL_CTX_use_certificate_file(ctx, CERTS_DIR "server.crt", SSL_FILETYPE_PEM);
    SSL_CTX_use_PrivateKey_file(ctx, CERTS_DIR "server.key", SSL_FILETYPE_PEM);
  } else {
    SSL_CTX_use_certificate_file(ctx, CERTS_DIR "client.crt", SSL_FILETYPE_PEM);
    SSL_CTX_use_PrivateKey_file(ctx, CERTS_DIR "client.key", SSL_FILETYPE_PEM);
  }

  SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, nullptr);
  SSL_CTX_load_verify_locations(ctx, CERTS_DIR "ca.crt", nullptr);

  return ctx;
}

void handle_connection(SSL *ssl) {

  char buf[1024];
  std::cout << "Starting connection handler for SSL session" << std::endl;

  std::thread send_thread([ssl]() {
    while (true) {
      std::string msg;
      std::cout << "Enter message: ";
      std::getline(std::cin, msg);

      if (msg == "quit") {
        std::cout << "Terminating send thread" << std::endl;
        break;
      }

      int bytes_sent = SSL_write(ssl, msg.c_str(), msg.size());
      if (bytes_sent <= 0) {
        int ssl_error = SSL_get_error(ssl, bytes_sent);
        std::cerr << "SSL_write failed with error: " << ssl_error << std::endl;
        ERR_print_errors_fp(stderr);
        break;
      }
      std::cout << "Sent " << bytes_sent << " bytes to server" << std::endl;
    }
  });

  while (true) {
    int bytes = SSL_read(ssl, buf, sizeof(buf) - 1);
    if (bytes > 0) {
      buf[bytes] = '\0';
      std::cout << "Received " << bytes << " bytes from server: " << buf
                << std::endl;
    } else if (bytes == 0) {
      std::cout << "Connection closed by peer" << std::endl;
      break;
    } else {
      int ssl_error = SSL_get_error(ssl, bytes);
      std::cerr << "SSL_read failed with error: " << ssl_error << std::endl;
      ERR_print_errors_fp(stderr);
      break;
    }
  }
  std::clog << "joining treads";
  send_thread.join();
  std::cout << "Connection handler finished" << std::endl;
}
